<?php

namespace Drupal\jsonapi_advanced_permissions\Routing;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Routing\RouteSubscriberBase;
use Symfony\Component\Routing\RouteCollection;

/**
 * Listens to the dynamic route events and alters JSON:API routes.
 */
class RouteSubscriber extends RouteSubscriberBase {

  /**
   * The config service.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * Constructs a RouteSubscriber object.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The config factory service.
   */
  public function __construct(ConfigFactoryInterface $config_factory) {
    $this->configFactory = $config_factory;
  }

  /**
   * {@inheritdoc}
   */
  protected function alterRoutes(RouteCollection $collection) {
    foreach ($collection->all() as $route_name => $route) {
      $defaults = $route->getDefaults();
      if (!empty($defaults['_is_jsonapi']) && !empty($defaults['resource_type'])) {
        $configParts = $this->configFactory->get('jsonapi_advanced_permissions.settings')->get('permissions') ?? [];
        foreach ($configParts as $configPart) {
          if ($route_name == 'jsonapi.' . $defaults['resource_type'] . '.' . $configPart) {
            $methods = $route->getMethods();
            foreach ($methods as $method) {
              $methodName = strtolower($method);
              $resourceType = $defaults['resource_type'];
              $resourceTypeName = str_replace('--', ' ', $resourceType);
              $resourceTypeName = str_replace('_', ' ', $resourceTypeName);
              $permissionName = str_replace('jsonapi.', '', $route_name);
              $permissionName = str_replace('--', '__', $permissionName);
              $permissionName = str_replace('.', ' ', $permissionName);
              $route->addRequirements([
                '_permission' => "jsonapi_advanced_permissions $methodName on $permissionName",
              ]);
            }
          }
        }
      }
    }
  }

}
